"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PendingOptimisticValueBrand;
(function (PendingOptimisticValueBrand) {
})(PendingOptimisticValueBrand = exports.PendingOptimisticValueBrand || (exports.PendingOptimisticValueBrand = {}));
var FulfilledOptimisticValueBrand;
(function (FulfilledOptimisticValueBrand) {
})(FulfilledOptimisticValueBrand = exports.FulfilledOptimisticValueBrand || (exports.FulfilledOptimisticValueBrand = {}));
var RejectedOptimisticValueBrand;
(function (RejectedOptimisticValueBrand) {
})(RejectedOptimisticValueBrand = exports.RejectedOptimisticValueBrand || (exports.RejectedOptimisticValueBrand = {}));
function extractID(value) {
    var match = /^__bicycle_optimistic_value_[0-9a-f]+_([0-9]+)__$/.exec(value);
    return parseInt(match[1], 10);
}
var OptimisticValueStore = /** @class */ (function () {
    function OptimisticValueStore(key, nextID) {
        this._fulfilled = new Map();
        this._rejected = new Map();
        // make it difficult to intentionally forge the optimistic values
        this._key =
            key ||
                Math.random()
                    .toString(16)
                    .substr(2);
        this._nextID = nextID || 0;
    }
    OptimisticValueStore.prototype.isOptimisticValue = function (value) {
        if (typeof value !== 'string') {
            return false;
        }
        var match = /^__bicycle_optimistic_value_([0-9a-f]+)_[0-9]+__$/.exec(value);
        if (!match)
            return false;
        return match[1] === this._key;
    };
    OptimisticValueStore.prototype.createValue = function () {
        var id = this._nextID++;
        return "__bicycle_optimistic_value_" + this._key + "_" + id + "__";
    };
    OptimisticValueStore.prototype.resolve = function (optimisticValue, value) {
        var id = extractID(optimisticValue);
        if (!this._fulfilled.has(id) && !this._rejected.has(id)) {
            this._fulfilled.set(id, value);
        }
    };
    OptimisticValueStore.prototype.reject = function (optimisticValue, err) {
        var id = extractID(optimisticValue);
        if (!this._fulfilled.has(id) && !this._rejected.has(id)) {
            this._rejected.set(id, err);
        }
    };
    OptimisticValueStore.prototype.isFulfilled = function (value) {
        return this._fulfilled.has(extractID(value));
    };
    OptimisticValueStore.prototype.isRejected = function (value) {
        return this._rejected.has(extractID(value));
    };
    OptimisticValueStore.prototype.getValue = function (value) {
        return this._fulfilled.get(extractID(value));
    };
    OptimisticValueStore.prototype.getError = function (value) {
        return this._rejected.get(extractID(value));
    };
    OptimisticValueStore.prototype.normalizeValue = function (value) {
        var store = this;
        var pendingKeys = [];
        var rejection = null;
        function recurse(value) {
            if (store.isOptimisticValue(value)) {
                if (store.isFulfilled(value)) {
                    return store.getValue(value);
                }
                else if (store.isRejected(value)) {
                    rejection = store.getError(value);
                }
                else {
                    pendingKeys.push(value);
                    return value;
                }
            }
            if (value && typeof value === 'object') {
                if (Array.isArray(value)) {
                    return value.map(recurse);
                }
                else {
                    var result_1 = {};
                    Object.keys(value).forEach(function (key) {
                        result_1[key] = recurse(value[key]);
                    });
                    return result_1;
                }
            }
            return value;
        }
        return { value: recurse(value), pendingKeys: pendingKeys, rejection: rejection };
    };
    return OptimisticValueStore;
}());
exports.default = OptimisticValueStore;
//# sourceMappingURL=OptimisticValueStore.js.map