import Cache, { CacheUpdate } from '../types/Cache';
import Query from '../types/Query';
import ServerPreparation from '../types/ServerPreparation';
import NetworkLayerInterface from '../types/NetworkLayerInterface';
import { QueryCacheResult } from '../utils/run-query-against-cache';
import NetworkLayer from '../network-layer';
import ErrorResult from '../types/ErrorResult';
import { OptimisticUpdateHandler } from './optimistic';
import { createNodeID } from '../types/NodeID';
import { BaseRootQuery, Mutation as TypedMutation } from '../typed-helpers/query';
export { NetworkLayer, NetworkLayerInterface as INetworkLayer, QueryCacheResult, createNodeID };
export declare type ClientOptions = {
    networkLayer?: NetworkLayerInterface;
    serverPreparation?: ServerPreparation;
    cacheTimeout?: number;
};
export interface Subscription {
    unsubscribe: () => void;
}
declare class Client<OptimisticUpdatersType = {
    [typeName: string]: {
        [mutationName: string]: OptimisticUpdateHandler;
    };
}> {
    private readonly _options;
    private _cache;
    private _optimisticCache;
    private readonly _optimisticValueStore;
    private readonly _queries;
    private readonly _queriesCount;
    private readonly _updateHandlers;
    private readonly _networkErrorHandlers;
    private readonly _mutationErrorHandlers;
    private readonly _queueRequestHandlers;
    private readonly _successfulResponseHandlers;
    private readonly _optimisticUpdaters;
    private readonly _request;
    constructor(options?: ClientOptions);
    private _onUpdate(fn);
    private _offUpdate(fn);
    _handleNetworkError(err: Error): void;
    _handleMutationError(err: Error): void;
    _handleQueueRequest(): void;
    _handleSuccessfulResponse(pendingMutations: number): void;
    _handleNewSession(data: Cache): void;
    _handleUpdate(data: void | CacheUpdate): void;
    private _updateOptimistCache();
    private _getQuery();
    private _updateQuery();
    private _addQuery(query);
    private _removeQuery(query);
    queryCache<TResult>(query: BaseRootQuery<TResult>): QueryCacheResult<TResult>;
    queryCache(query: Query): QueryCacheResult<any>;
    query<TResult>(query: BaseRootQuery<TResult>): Promise<TResult>;
    query(query: Query): Promise<any>;
    defineOptimisticUpdaters(updates: OptimisticUpdatersType): void;
    update<TResult>(mutation: TypedMutation<TResult>): Promise<TResult>;
    update(method: string, args: any, optimisticUpdate?: OptimisticUpdateHandler): Promise<any>;
    subscribe<TResult>(query: BaseRootQuery<TResult>, fn: (result: TResult, loaded: boolean, errors: ReadonlyArray<string>, errorDetails: ReadonlyArray<ErrorResult>) => any): Subscription;
    subscribe(query: Query, fn: (result: any, loaded: boolean, errors: ReadonlyArray<string>, errorDetails: ReadonlyArray<ErrorResult>) => any): Subscription;
    subscribeToNetworkErrors(fn: (err: Error) => any): Subscription;
    subscribeToMutationErrors(fn: (err: Error) => any): Subscription;
    subscribeToQueueRequest(fn: () => any): Subscription;
    subscribeToSuccessfulResponse(fn: (pendingMutations: number) => any): Subscription;
}
export default Client;
