"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var cuid = require("cuid");
var Mutation = /** @class */ (function () {
    function Mutation(method, args, optimisticUpdate, optimisticValueStore) {
        var _this = this;
        this._optimisticValuesSuplied = {};
        this._pending = true;
        this._rejected = false;
        this._getOptimisticValue = function (name) {
            return (_this._optimisticValuesSuplied[name] ||
                (_this._optimisticValuesSuplied[name] = _this._optimisticValueStore.createValue()));
        };
        this._optimisticValueStore = optimisticValueStore;
        var result = optimisticValueStore.normalizeValue(args);
        this._optimisticValuesRequired = result.pendingKeys;
        this.mutation = {
            id: cuid(),
            method: method,
            args: result.value,
        };
        this._structuredMutation = {
            objectName: method.split('.')[0],
            methodName: method.split('.')[1],
            args: args,
        };
        this._optimisticUpdateHandler = optimisticUpdate;
        this._result = new Promise(function (resolve, reject) {
            _this._resolve = resolve;
            _this._reject = reject;
        });
        this.updateStatus();
    }
    Mutation.prototype.updateStatus = function () {
        if (!this._pending)
            return;
        if (this._optimisticValuesRequired.length === 0)
            return;
        var result = this._optimisticValueStore.normalizeValue(this.mutation.args);
        this.mutation.args = result.value;
        this._structuredMutation.args = result.value;
        if (result.rejection) {
            this.reject(result.rejection);
        }
        this._optimisticValuesRequired = result.pendingKeys;
    };
    Mutation.prototype.isPending = function () {
        return this._pending;
    };
    Mutation.prototype.isRejected = function () {
        return this._rejected;
    };
    Mutation.prototype.isBlocked = function () {
        return !this._rejected && this._optimisticValuesRequired.length !== 0;
    };
    Mutation.prototype.applyOptimistic = function (cache) {
        var handler = this._optimisticUpdateHandler;
        if (!handler)
            return;
        handler(this._structuredMutation, cache, this._getOptimisticValue);
    };
    Mutation.prototype.getResult = function () {
        return this._result;
    };
    Mutation.prototype.resolve = function (result) {
        var _this = this;
        this._pending = false;
        Object.keys(this._optimisticValuesSuplied).forEach(function (name) {
            var v = _this._optimisticValuesSuplied[name];
            if (v) {
                if (result && typeof result === 'object' && name in result) {
                    _this._optimisticValueStore.resolve(v, result[name]);
                }
                else {
                    _this._optimisticValueStore.reject(v, new Error('Could not resolve missing optimistic value "' + name + '"'));
                }
            }
        });
        this._resolve(result);
    };
    Mutation.prototype.reject = function (err) {
        var _this = this;
        this._pending = false;
        this._rejected = true;
        Object.keys(this._optimisticValuesSuplied).forEach(function (name) {
            var v = _this._optimisticValuesSuplied[name];
            _this._optimisticValueStore.reject(v, err);
        });
        this._reject(err);
    };
    return Mutation;
}());
exports.default = Mutation;
//# sourceMappingURL=mutation.js.map