"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var NodeID_1 = require("../types/NodeID");
var ErrorResult_1 = require("../types/ErrorResult");
var stringify = require('stable-stringify');
function extractCacheData(data, cache, resultCache) {
    if (NodeID_1.isID(data)) {
        return new BaseCache(data, cache, resultCache);
    }
    if (ErrorResult_1.isErrorResult(data)) {
        return undefined;
    }
    if (Array.isArray(data)) {
        return data.map(function (v) { return extractCacheData(v, cache, resultCache); });
    }
    if (data && typeof data === 'object') {
        var result_1 = {};
        Object.keys(data).forEach(function (key) { return (result_1[key] = extractCacheData(data[key], cache, resultCache)); });
        return result_1;
    }
    return data;
}
function packageCacheData(data) {
    if (Array.isArray(data)) {
        return data.map(function (v) { return packageCacheData(v); });
    }
    if (data && typeof data === 'object') {
        if (data instanceof BaseCache && NodeID_1.isID(data.id)) {
            return data.id;
        }
        var result_2 = {};
        Object.keys(data).forEach(function (key) { return (result_2[key] = packageCacheData(data[key])); });
        return result_2;
    }
    return data;
}
var BaseCache = /** @class */ (function () {
    function BaseCache(id, cache, resultCache) {
        this._resultData = undefined;
        this.id = id;
        this._data = (cache[id.n] && cache[id.n][id.i]) || {};
        this._resultData =
            (resultCache[id.n] && resultCache[id.n][id.i]) || undefined;
        this._cache = cache;
        this._resultCache = resultCache;
    }
    BaseCache.prototype.get = function (name, args) {
        var key = args === undefined ? name : name + '(' + stringify(args) + ')';
        var result = this._resultData && this._resultData[key];
        return extractCacheData(result === undefined ? this._data[key] : result, this._cache, this._resultCache);
    };
    BaseCache.prototype.set = function (name, args, value) {
        var v = value === undefined ? args : value;
        var a = value === undefined ? undefined : args;
        this._resultData = NodeID_1.getNode(this._resultCache, this.id);
        this._resultData[a === undefined ? name : name + '(' + stringify(a) + ')'] = packageCacheData(v);
        return this;
    };
    BaseCache.prototype.getObject = function (typeName, id) {
        var i = NodeID_1.createNodeID(typeName, id);
        return new BaseCache(i, this._cache, this._resultCache);
    };
    return BaseCache;
}());
exports.BaseCache = BaseCache;
//# sourceMappingURL=optimistic.js.map