"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var diff_queries_1 = require("../utils/diff-queries");
var create_error_1 = require("../utils/create-error");
var Request_1 = require("../types/Request");
function wait(timeout) {
    return new Promise(function (resolve) { return setTimeout(resolve, timeout); });
}
var RequestStatus;
(function (RequestStatus) {
    RequestStatus[RequestStatus["IDLE"] = 0] = "IDLE";
    RequestStatus[RequestStatus["REQUEST_QUEUED"] = 1] = "REQUEST_QUEUED";
    RequestStatus[RequestStatus["REQUEST_IN_FLIGHT"] = 2] = "REQUEST_IN_FLIGHT";
})(RequestStatus || (RequestStatus = {}));
function assert(fact) {
    if (!fact) {
        throw new Error('Assertion violated');
    }
}
function assertUnreachable(p) {
    throw new Error('Expected code to be unreachable');
}
var SessionStatus;
(function (SessionStatus) {
    SessionStatus[SessionStatus["NEW"] = 0] = "NEW";
    SessionStatus[SessionStatus["EXPIRED"] = 1] = "EXPIRED";
    SessionStatus[SessionStatus["NORMAL"] = 2] = "NORMAL";
})(SessionStatus || (SessionStatus = {}));
var RequestBatcher = /** @class */ (function () {
    function RequestBatcher(networkLayer, serverPreparation, handlers) {
        this._networkLayer = networkLayer;
        this._handlers = handlers;
        this._session = serverPreparation
            ? {
                status: 2 /* NORMAL */,
                sessionID: serverPreparation.s,
                version: serverPreparation.v,
                serverQuery: serverPreparation.q,
            }
            : { status: 0 /* NEW */ };
        this._pendingMutations = [];
        this._localQuery = serverPreparation ? serverPreparation.q : {};
        this._status = 0 /* IDLE */;
    }
    RequestBatcher.prototype.updateQuery = function (query) {
        this._localQuery = query;
        this._request();
    };
    RequestBatcher.prototype.runMutation = function (mutation) {
        this._pendingMutations.push(mutation);
        this._request();
        this._handlers._handleUpdate(undefined);
        return mutation.getResult();
    };
    RequestBatcher.prototype.getPendingMutations = function () {
        return this._pendingMutations;
    };
    // make request, using any in-flight requests
    RequestBatcher.prototype._request = function () {
        switch (this._status) {
            case 0 /* IDLE */:
                this._queueRequest();
                break;
            case 1 /* REQUEST_QUEUED */:
                break;
            case 2 /* REQUEST_IN_FLIGHT */:
                this._status = 1 /* REQUEST_QUEUED */;
                break;
            default:
                assertUnreachable(this._status);
        }
    };
    // queue a new request
    RequestBatcher.prototype._queueRequest = function () {
        return __awaiter(this, void 0, void 0, function () {
            var errors, localQuery, mutations, i, response, ex_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this._handlers._handleQueueRequest();
                        assert(this._status === 0 /* IDLE */);
                        this._status = 1 /* REQUEST_QUEUED */;
                        // Set up the request but don't fire it for 30ms, this batches up requests
                        return [4 /*yield*/, wait(30)];
                    case 1:
                        // Set up the request but don't fire it for 30ms, this batches up requests
                        _a.sent();
                        errors = 0;
                        _a.label = 2;
                    case 2:
                        if (!(this._status === 1 /* REQUEST_QUEUED */)) return [3 /*break*/, 8];
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 7]);
                        this._status = 2 /* REQUEST_IN_FLIGHT */;
                        localQuery = this._localQuery;
                        mutations = [];
                        for (i = 0; i < this._pendingMutations.length &&
                            !this._pendingMutations[i].isBlocked(); i++) {
                            if (this._pendingMutations[i].isPending()) {
                                mutations.push(this._pendingMutations[i]);
                            }
                        }
                        return [4 /*yield*/, this._fireRequest(localQuery, mutations)];
                    case 4:
                        response = _a.sent();
                        switch (response.k) {
                            // status could be normal or expired before getting this response
                            case 0 /* EXPIRED */:
                                console.warn('session expired, restoring session');
                                this._session = {
                                    status: 1 /* EXPIRED */,
                                    sessionID: this._session
                                        .sessionID,
                                };
                                if (response.m) {
                                    // we keep resolved mutations because we haven't run a query
                                    // yet so we still want their optimitic updates
                                    this._handleMutationResults(mutations, response.m, true);
                                }
                                this._handlers._handleSuccessfulResponse(this._pendingMutations.length);
                                // updating clears any rejected mutation's optimstic updates
                                this._handlers._handleUpdate(undefined);
                                this._status = 1 /* REQUEST_QUEUED */;
                                break;
                            // status is always new session before getting this response
                            case 1 /* NEW_SESSION */:
                                this._session = {
                                    status: 2 /* NORMAL */,
                                    sessionID: response.s,
                                    version: response.v,
                                    serverQuery: localQuery,
                                };
                                this._handlers._handleSuccessfulResponse(this._pendingMutations.length);
                                this._handlers._handleNewSession(response.d);
                                if (this._pendingMutations.length) {
                                    // send a request with mutations now we have a session
                                    this._status = 1 /* REQUEST_QUEUED */;
                                }
                                break;
                            // status is always expired before getting this response
                            case 3 /* RESTORE */:
                                this._session = {
                                    status: 2 /* NORMAL */,
                                    sessionID: this._session.sessionID,
                                    version: response.v,
                                    serverQuery: localQuery,
                                };
                                if (response.m) {
                                    this._handleMutationResults(mutations, response.m, false);
                                }
                                this._handlers._handleSuccessfulResponse(this._pendingMutations.length);
                                this._handlers._handleNewSession(response.d);
                                break;
                            // status is always normal before getting this response
                            case 2 /* UPDATE */:
                                this._session = {
                                    status: 2 /* NORMAL */,
                                    sessionID: this._session.sessionID,
                                    version: response.v,
                                    serverQuery: localQuery,
                                };
                                if (response.m) {
                                    this._handleMutationResults(mutations, response.m, false);
                                }
                                this._handlers._handleSuccessfulResponse(this._pendingMutations.length);
                                this._handlers._handleUpdate(response.d);
                                break;
                        }
                        errors = 0;
                        return [3 /*break*/, 7];
                    case 5:
                        ex_1 = _a.sent();
                        errors++;
                        this._status === 1 /* REQUEST_QUEUED */;
                        return [4 /*yield*/, wait(1000 * (errors + Math.random()))];
                    case 6:
                        _a.sent();
                        return [3 /*break*/, 7];
                    case 7: return [3 /*break*/, 2];
                    case 8:
                        assert(this._status === 2 /* REQUEST_IN_FLIGHT */);
                        this._status = 0 /* IDLE */;
                        return [2 /*return*/];
                }
            });
        });
    };
    RequestBatcher.prototype._fireRequest = function (localQuery, mutations) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (this._session.status) {
                    case 0 /* NEW */:
                        return [2 /*return*/, this._networkLayer.send(Request_1.createNewSessionRequest(localQuery))];
                    case 1 /* EXPIRED */:
                        return [2 /*return*/, this._networkLayer.send(Request_1.createRestoreRequest(this._session.sessionID, localQuery, mutations))];
                    case 2 /* NORMAL */:
                        return [2 /*return*/, this._networkLayer.send(Request_1.createUpdateRequest(this._session.sessionID, this._session.version, diff_queries_1.default(this._session.serverQuery, localQuery), mutations))];
                }
                return [2 /*return*/];
            });
        });
    };
    RequestBatcher.prototype._handleMutationResults = function (mutations, mutationResults, keepResolvedMutations) {
        var _this = this;
        if (keepResolvedMutations === void 0) { keepResolvedMutations = false; }
        mutations.forEach(function (mutation, i) {
            if (i >= mutationResults.length) {
                return;
            }
            var result = mutationResults[i];
            if (result.s) {
                mutation.resolve(result.v);
            }
            else {
                var err = create_error_1.default(result.v.message, {
                    data: result.v.data,
                    code: result.v.code,
                    mutation: mutation.mutation,
                });
                _this._handlers._handleMutationError(err);
                mutation.reject(err);
            }
        });
        this._pendingMutations = this._pendingMutations.filter(function (mutation) {
            mutation.updateStatus();
            return (mutation.isPending() ||
                (keepResolvedMutations && !mutation.isRejected()));
        });
    };
    return RequestBatcher;
}());
exports.default = RequestBatcher;
//# sourceMappingURL=request-batcher.js.map