"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var suggest_match_1 = require("../utils/suggest-match");
var type_name_from_value_1 = require("../utils/type-name-from-value");
var Type = /** @class */ (function () {
    function Type(typeName, validate) {
        this.typeName = typeName;
        this._validate = validate;
    }
    Type.prototype.validate = function (value, name) {
        if (name === void 0) { name = 'value'; }
        if (!this._validate(value, name)) {
            throw new Error('Expected ' +
                name +
                ' to be a ' +
                this.typeName +
                ' but got ' +
                type_name_from_value_1.default(value));
        }
        return value;
    };
    Type.prototype.or = function (t) {
        var _this = this;
        return new Type(this.typeName + ' | ' + t.typeName, function (value, context) {
            return _this._validate(value, context) || t._validate(value, context);
        });
    };
    return Type;
}());
exports.Type = Type;
exports.Fn = new Type('function', function (v) { return typeof v === 'function'; });
exports.String = new Type('string', function (v) { return typeof v === 'string'; });
exports.Void = new Type('undefined', function (v) { return v === undefined; });
exports.Literal = function (value) {
    return new Type(JSON.stringify(value), function (v) { return v === value; });
};
exports.ArrayOf = function (elementType) {
    return new Type((elementType.typeName.indexOf(' ') !== -1
        ? '(' + elementType.typeName + ')'
        : elementType.typeName) + '[]', function (v, context) {
        return Array.isArray(v) && v.every(function (v) { return elementType._validate(v, context); });
    });
};
exports.ObjectKeys = function (keys) {
    return new Type('{' + keys.join(', ') + '}', function (v, context) {
        if (!(v && typeof v === 'object')) {
            return false;
        }
        Object.keys(v).forEach(function (key) {
            if (keys.indexOf(key) === -1) {
                var suggestion = suggest_match_1.default(keys, key);
                throw new Error("Invalid key \"" + key + "\" in " + context + suggestion);
            }
        });
        return true;
    });
};
exports.AnyObject = new Type('Object', function (v) {
    return v && typeof v === 'object' && !Array.isArray(v);
});
//# sourceMappingURL=TypeAssertions.js.map