"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var suggest_match_1 = require("../utils/suggest-match");
function getTypeFromString(strType, context, typeNames) {
    if (strType[strType.length - 1] === '?') {
        return {
            kind: "Union" /* Union */,
            elements: [
                getType(strType.substr(0, strType.length - 1), context, typeNames),
                { kind: "Null" /* Null */ },
                { kind: "Void" /* Void */ },
            ],
        };
    }
    if (strType[strType.length - 2] === '[' &&
        strType[strType.length - 1] === ']') {
        return {
            kind: "List" /* List */,
            element: getType(strType.substr(0, strType.length - 2), context, typeNames),
        };
    }
    var contextStr = context ? ' for ' + context : '';
    assert(/^[A-Za-z0-9]+$/.test(strType), "Expected type name to match [A-Za-z0-9]+ but got '" + strType + "'" + contextStr);
    switch (strType) {
        case 'boolean':
            return { kind: "Boolean" /* Boolean */ };
        case 'string':
            return { kind: "String" /* String */ };
        case 'number':
            return { kind: "Number" /* Number */ };
        case 'void':
            return { kind: "Void" /* Void */ };
        case 'null':
            return { kind: "Null" /* Null */ };
        case 'any':
            return { kind: "Any" /* Any */ };
    }
    if (typeNames.indexOf(strType) === -1) {
        var suggestion = suggest_match_1.default(typeNames, strType);
        throw new Error(context + " refers to " + strType + ", but there is no type by that name" + suggestion);
    }
    return { kind: "Named" /* Named */, name: strType };
}
function getTypeFromObject(objType, context, typeNames) {
    var properties = {};
    Object.keys(objType).forEach(function (key) {
        properties[key] = getType(objType[key], context + '.' + key, typeNames);
    });
    return { kind: "Object" /* Object */, properties: properties };
}
function getType(type, context, typeNames) {
    if (typeof type === 'string') {
        return getTypeFromString(type, context, typeNames);
    }
    else if (type && typeof type === 'object' && !Array.isArray(type)) {
        return getTypeFromObject(type, context, typeNames);
    }
    else {
        throw new Error(context + " has an invalid type. Types must be strings or objects.");
    }
}
exports.default = getType;
//# sourceMappingURL=get-type.js.map