"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var normalize_object_1 = require("./normalize-object");
var normalize_scalar_1 = require("./normalize-scalar");
var TA = require("./TypeAssertions");
function loadSchema(input) {
    var i = TA.ObjectKeys(['scalars', 'objects']).validate(input, 'input');
    var objects = TA.ArrayOf(TA.AnyObject).validate(i.objects, 'input.objects');
    var scalars = TA.Void.or(TA.ArrayOf(TA.AnyObject)).validate(i.scalars, 'input.objects');
    var types = { Root: null };
    var typeNames = [];
    var rootObject = null;
    objects.forEach(function (Type) {
        var typeName = TA.String.validate(Type.name, 'Type.name');
        if (typeName === 'Root') {
            if (rootObject !== null) {
                throw new Error("Duplicate Root Object.  You can only have one Root Object.");
            }
            rootObject = Type;
        }
        else if (typeName) {
            if (typeNames.indexOf(typeName) !== -1) {
                throw new Error("Duplicate Object, \"" + typeName + "\".  Each object & scalar must have a unique name.");
            }
            typeNames.push(typeName);
        }
    });
    if (scalars) {
        scalars.forEach(function (Scalar) {
            var scalarName = TA.String.validate(Scalar.name, 'Scalar.name');
            if (typeNames.indexOf(scalarName) !== -1) {
                throw new Error("Duplicate Scalar, \"" + scalarName + "\".  Each object & scalar must have a unique name.");
            }
            else if (scalarName === 'Root') {
                throw new Error('You cannot have a scalar called Root');
            }
            else if (scalarName) {
                typeNames.push(scalarName);
            }
        });
    }
    if (scalars) {
        scalars.forEach(function (Scalar) {
            var scalarName = TA.String.validate(Scalar.name, 'Scalar.name');
            types[scalarName] = normalize_scalar_1.default(Scalar, typeNames);
        });
    }
    // objects have `id`s and a collection of `fields`
    objects.forEach(function (Type) {
        var typeName = TA.String.validate(Type.name, 'Type.name');
        types[typeName] = normalize_object_1.default(Type, typeNames);
    });
    if (!types.Root) {
        throw new Error('You must provide a Root object');
    }
    return types;
}
exports.default = loadSchema;
function loadSchemaFromFiles(dirname) {
    dirname = dirname.replace(/(\\|\/)$/, '');
    var schema = { objects: [], scalars: [] };
    fs_1.readdirSync(dirname + '/objects').forEach(function (filename) {
        var t = require(dirname + '/objects/' + filename);
        if (t.default)
            t = t.default;
        schema.objects.push(t);
    });
    var scalars = null;
    try {
        scalars = fs_1.readdirSync(dirname + '/scalars');
    }
    catch (ex) {
        if (ex.code !== 'ENOENT')
            throw ex;
    }
    if (scalars) {
        scalars.forEach(function (filename) {
            var t = require(dirname + '/scalars/' + filename);
            if (t.default)
                t = t.default;
            schema.scalars.push(t);
        });
    }
    return loadSchema(schema);
}
exports.loadSchemaFromFiles = loadSchemaFromFiles;
//# sourceMappingURL=index.js.map