"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var normalize_mutation_1 = require("./normalize-mutation");
var ta = require("./TypeAssertions");
var validate_1 = require("../runner/validate");
function normalizeMutations(mutations, typeName, typeNames, fields) {
    var result = {};
    var m = ta.Void.or(ta.AnyObject).validate(mutations, typeName + '.mutations');
    if (m === undefined) {
        return {};
    }
    Object.keys(m).forEach(function (mutationName) {
        var mutation = m[mutationName];
        if (mutation === undefined) {
            return;
        }
        assert(/^[A-Za-z]+$/.test(mutationName), "Expected " + typeName + "'s mutation names to match [A-Za-z0-9]+ but got '" + mutationName + "'");
        if (mutationName === 'set') {
            var set_1 = ta.Fn.validate(m.set, 'Set Mutation');
            result.set = {
                kind: "Mutation" /* Mutation */,
                name: typeName + '.set',
                description: 'Set a property on the given ' + typeName,
                argType: {
                    kind: "Object" /* Object */,
                    properties: {
                        id: {
                            kind: "Union" /* Union */,
                            elements: [{ kind: "String" /* String */ }, { kind: "Number" /* Number */ }],
                        },
                        field: {
                            kind: "Union" /* Union */,
                            elements: Object.keys(fields).map(function (n) { return ({
                                kind: "Literal" /* Literal */,
                                value: n,
                            }); }),
                        },
                        value: {
                            kind: "Any" /* Any */,
                        },
                    },
                },
                resultType: { kind: "Void" /* Void */ },
                auth: 'public',
                resolve: function (args, ctx, mCtx) {
                    validate_1.validateArg(fields[args.field].resultType, args.value, mCtx.schema);
                    return set_1(args, ctx, mCtx);
                },
            };
        }
        else {
            result[mutationName] = normalize_mutation_1.default(mutation, typeName, mutationName, typeNames);
        }
    });
    return result;
}
exports.default = normalizeMutations;
//# sourceMappingURL=normalize-mutations.js.map