"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var type_name_from_value_1 = require("../utils/type-name-from-value");
var normalize_fields_1 = require("./normalize-fields");
var normalize_mutations_1 = require("./normalize-mutations");
var TA = require("./TypeAssertions");
function normalizeObject(Type, typeNames) {
    var typeName = TA.String.validate(TA.AnyObject.validate(Type, 'ObjectType').name, 'ObjectType.name');
    var t = TA.ObjectKeys([
        'name',
        'id',
        'description',
        'fields',
        'mutations',
    ]).validate(Type, typeName);
    assert(/^[A-Za-z]+$/.test(typeName), "Expected ObjectType.name to match [A-Za-z]+ but got '" + typeName + "'");
    var id = TA.Void.or(TA.Fn).validate(t.id, typeName + '.id');
    var description = TA.Void.or(TA.String).validate(t.description, typeName + '.description');
    assert(typeName !== 'Root' || id === undefined, "The Root object always has an ID of \"root\".  You don't need to specify an id function yourself");
    var idGetter = id;
    if (!idGetter) {
        if (typeName === 'Root') {
            idGetter = function () { return 'root'; };
        }
        else {
            idGetter = function (node) {
                if (typeof node.id !== 'string' && typeof node.id !== 'number') {
                    if (node.id === undefined || node.id === null) {
                        throw new Error('Node of type ' + typeName + ' does not have an id');
                    }
                    else {
                        throw new Error('Expected node of type ' +
                            typeName +
                            ' to have either a string or number for the "id" but got "' +
                            type_name_from_value_1.default(node.id) +
                            '"');
                    }
                }
                return node.id;
            };
        }
    }
    var fields = normalize_fields_1.default(TA.AnyObject.validate(t.fields, typeName + '.fields'), typeName, typeNames);
    var m = TA.Void.or(TA.AnyObject).validate(t.mutations, typeName + '.mutations');
    var mutations = m ? normalize_mutations_1.default(m, typeName, typeNames, fields) : {};
    return {
        kind: "NodeType" /* NodeType */,
        name: typeName,
        id: idGetter,
        description: description,
        fields: fields,
        mutations: mutations,
        matches: function (value) {
            // untyped mode does not support unions
            return true;
        },
    };
}
exports.default = normalizeObject;
//# sourceMappingURL=normalize-object.js.map