"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var character_parser_1 = require("character-parser");
var util_1 = require("util");
var create_error_1 = require("../utils/create-error");
// TODO: actually use this as a fallback when JSON.parse fails
// parse from string of the form (name: "value" ...) into a JSON object (to match the args format for updates)
function parseArgs(args) {
    var result = {};
    var fullArgsString = args;
    var state = 'key';
    if (args[0] !== '(') {
        throw new Error('Legacy args must start with an open parenthesis');
    }
    args = args.trim().substr(1); // ignore initial open bracket
    var currentKey = '';
    var currentValue = '';
    var cpState = character_parser_1.defaultState();
    while (args.length) {
        switch (state) {
            case 'key':
                if (args[0] === ':') {
                    state = 'value';
                    cpState = character_parser_1.defaultState();
                    currentKey = currentKey.trim();
                    args = args.substr(1);
                    if (!currentKey) {
                        throw create_error_1.default("Argument name cannot be empty string, full string was \"" + fullArgsString + "\"", {
                            exposeProd: true,
                            code: 'ERROR_PARSING_ARGS',
                            data: { fullArgsString: fullArgsString },
                        });
                    }
                }
                else if (args[0] === ')' && !currentKey.trim()) {
                    state = 'terminated';
                    args = args.substr(1);
                }
                else {
                    currentKey += args[0];
                    args = args.substr(1);
                }
                break;
            case 'value':
                if (cpState.isNesting() || (args[0] !== ')' && args[0] !== ',')) {
                    currentValue += args[0];
                    cpState = character_parser_1.parseChar(args[0], cpState);
                    args = args.substr(1);
                }
                else if (args[0] === ')') {
                    result[currentKey] = parseValue(currentValue.trim(), currentKey);
                    state = 'terminated';
                    args = args.substr(1);
                }
                else {
                    result[currentKey] = parseValue(currentValue.trim(), currentKey);
                    state = 'key';
                    currentKey = '';
                    currentValue = '';
                    args = args.substr(1);
                }
                break;
            case 'terminated':
                throw create_error_1.default("Closing bracket was reached before end of arguments, full string was \"" + fullArgsString + "\"", {
                    exposeProd: true,
                    code: 'ERROR_PARSING_ARGS',
                    data: { fullArgsString: fullArgsString },
                });
        }
    }
    if (state !== 'terminated') {
        throw create_error_1.default("End of args string reached with no closing bracket, full string was \"" + fullArgsString + "\"", { exposeProd: true, code: 'ERROR_PARSING_ARGS', data: { fullArgsString: fullArgsString } });
    }
    return result;
}
exports.default = parseArgs;
function parseValue(value, argName) {
    try {
        return value === 'undefined' || !value ? null : JSON.parse(value);
    }
    catch (ex) {
        throw create_error_1.default("Could not parse arg \"" + argName + " with value " + util_1.inspect(value) + ", make sure the argument values are always valid JSON strings.", { exposeProd: true, code: 'ERROR_PARSING_ARG', data: { argName: argName, value: value } });
    }
}
//# sourceMappingURL=legacyArgParser.js.map