"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function matchesType(type, value, schema, allowNodes) {
    switch (type.kind) {
        case "Boolean" /* Boolean */:
            return typeof value === 'boolean';
        case "List" /* List */:
            return (Array.isArray(value) &&
                value.every(function (v) { return matchesType(type.element, v, schema, allowNodes); }));
        case "Literal" /* Literal */:
            return value === type.value;
        case "Null" /* Null */:
            return value === null;
        case "Number" /* Number */:
            return typeof value === 'number';
        case "Object" /* Object */:
            return (value &&
                typeof value === 'object' &&
                Object.keys(type.properties).every(function (p) {
                    return matchesType(type.properties[p], value[p], schema, allowNodes);
                }) &&
                Object.keys(value).every(function (p) {
                    return Object.prototype.hasOwnProperty.call(type.properties, p);
                }));
        case "String" /* String */:
            return typeof value === 'string';
        case "Union" /* Union */:
            return type.elements.some(function (e) { return matchesType(e, value, schema, allowNodes); });
        case "Void" /* Void */:
            return value === undefined;
        case "Any" /* Any */:
            return true;
        case "Promise" /* Promise */:
            throw new Error('Promises are not supported in this location');
        case "Named" /* Named */:
            return matchesNamedType(type, value, schema, allowNodes);
    }
}
exports.default = matchesType;
function matchesNamedType(type, value, schema, allowNodes) {
    var t = schema[type.name];
    switch (t.kind) {
        case "NodeType" /* NodeType */:
            return allowNodes && t.matches(value) === true;
        case "Scalar" /* Scalar */:
            return (
            // never allow nodes in the base type
            matchesType(t.baseType, value, schema, false) &&
                t.validate(value) === true);
    }
}
//# sourceMappingURL=matchesType.js.map