"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var matchesType_1 = require("./matchesType");
var validate_1 = require("./validate");
var type_name_from_definition_1 = require("../utils/type-name-from-definition");
var type_name_from_value_1 = require("../utils/type-name-from-value");
var create_error_1 = require("../utils/create-error");
var runQuery_1 = require("./runQuery");
function getError(type, value) {
    var expected = type_name_from_definition_1.default(type);
    if (process.env.NODE_ENV !== 'production') {
        var actual = type_name_from_value_1.default(value);
        return create_error_1.default("Expected result to be of type \"" + expected + "\" but got \"" + actual + "\"", { exposeProd: false, code: 'INVALID_RESULT_TYPE', data: { value: value, expected: expected } });
    }
    return create_error_1.default("Expected result to be of type \"" + expected + "\"", {
        exposeProd: true,
        code: 'INVALID_RESULT_TYPE',
        data: { expected: expected },
    });
}
function hasNamedType(type) {
    if (type._hasNamedType !== undefined) {
        return type._hasNamedType;
    }
    var result = false;
    switch (type.kind) {
        case "Union" /* Union */:
            result = type.elements.some(hasNamedType);
            break;
        case "List" /* List */:
            result = hasNamedType(type.element);
            break;
        case "Object" /* Object */:
            result = Object.keys(type.properties).some(function (key) {
                return hasNamedType(type.properties[key]);
            });
            break;
        case "Named" /* Named */:
            result = true;
            break;
        default:
            result = false;
            break;
    }
    type._hasNamedType = result;
    return result;
}
function resolveFieldResult(type, value, subQuery, qCtx) {
    return __awaiter(this, void 0, void 0, function () {
        var _this = this;
        var _a, matchingElements, result_1, s;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    if (!hasNamedType(type)) return [3 /*break*/, 8];
                    _a = type.kind;
                    switch (_a) {
                        case "Union" /* Union */: return [3 /*break*/, 1];
                        case "List" /* List */: return [3 /*break*/, 4];
                        case "Object" /* Object */: return [3 /*break*/, 5];
                        case "Named" /* Named */: return [3 /*break*/, 7];
                    }
                    return [3 /*break*/, 8];
                case 1:
                    if (type.elements.some(function (e) {
                        return !hasNamedType(e) && matchesType_1.default(type, value, qCtx.schema, false);
                    })) {
                        return [2 /*return*/, value];
                    }
                    matchingElements = type.elements.filter(function (e) { return hasNamedType(e) && matchesType_1.default(type, value, qCtx.schema, true); });
                    if (!(matchingElements.length === 1)) return [3 /*break*/, 3];
                    return [4 /*yield*/, resolveFieldResult(matchingElements[0], value, subQuery, qCtx)];
                case 2: return [2 /*return*/, _b.sent()];
                case 3:
                    if (matchingElements.length === 0) {
                        // it did not match any of the types in the union
                        throw getError(type, value);
                    }
                    else {
                        // it mached more tahn one of the types in the union
                        throw create_error_1.default("The result was ambiguous. It could be more than one of the different elements in the union \"" + type_name_from_definition_1.default(type) + "\"", {
                            exposeProd: true,
                            code: 'INVALID_RESULT_TYPE',
                            data: { expected: type_name_from_definition_1.default(type) },
                        });
                    }
                    _b.label = 4;
                case 4:
                    if (!Array.isArray(value)) {
                        throw getError(type, value);
                    }
                    return [2 /*return*/, Promise.all(value.map(function (v) { return __awaiter(_this, void 0, void 0, function () {
                            var result;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, resolveFieldResult(type.element, v, subQuery, qCtx)];
                                    case 1:
                                        result = _a.sent();
                                        if (Array.isArray(result)) {
                                            return [2 /*return*/, result];
                                        }
                                        return [2 /*return*/, result];
                                }
                            });
                        }); }))];
                case 5:
                    if (!value ||
                        typeof value !== 'object' ||
                        !Object.keys(value).every(function (p) {
                            return Object.prototype.hasOwnProperty.call(type.properties, p);
                        })) {
                        throw getError(type, value);
                    }
                    result_1 = {};
                    return [4 /*yield*/, Promise.all(Object.keys(type.properties).map(function (p) { return __awaiter(_this, void 0, void 0, function () {
                            var _a, _b;
                            return __generator(this, function (_c) {
                                switch (_c.label) {
                                    case 0:
                                        _a = result_1;
                                        _b = p;
                                        return [4 /*yield*/, resolveFieldResult(type.properties[p], value[p], subQuery, qCtx)];
                                    case 1:
                                        _a[_b] = _c.sent();
                                        return [2 /*return*/];
                                }
                            });
                        }); }))];
                case 6:
                    _b.sent();
                    return [2 /*return*/, result_1];
                case 7:
                    s = qCtx.schema[type.name];
                    if (s.kind == "Scalar" /* Scalar */) {
                        validate_1.validateResult(s.baseType, value, qCtx.schema);
                        if (s.validate(value) !== true) {
                            throw getError(type, value);
                        }
                        return [2 /*return*/, value];
                    }
                    if (s.matches(value) !== true) {
                        throw create_error_1.default('Expected subQuery to be an Object but got ' +
                            type_name_from_value_1.default(subQuery) +
                            ' while getting ' +
                            type.name, {});
                    }
                    if (!subQuery || typeof subQuery !== 'object') {
                        throw create_error_1.default('Expected subQuery to be an Object but got ' +
                            type_name_from_value_1.default(subQuery) +
                            ' while getting ' +
                            type.name, {});
                    }
                    return [2 /*return*/, runQuery_1.default(s, value, subQuery, qCtx)];
                case 8:
                    validate_1.validateResult(type, value, qCtx.schema);
                    return [2 /*return*/, value];
            }
        });
    });
}
exports.default = resolveFieldResult;
//# sourceMappingURL=resolveFieldResult.js.map