"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validate_1 = require("./validate");
var error_reporting_1 = require("../error-reporting");
var AccessDeniedType;
(function (AccessDeniedType) {
})(AccessDeniedType || (AccessDeniedType = {}));
var ACCESS_DENIED = {};
function isAccessDenied(v) {
    return v === ACCESS_DENIED;
}
function runMutation(mutation, arg, mCtx) {
    return Promise.resolve(null)
        .then(function () {
        validate_1.validateArg(mutation.argType, arg, mCtx.schema);
        return mutation.auth === 'public'
            ? true
            : mutation.auth(arg, mCtx.context, mCtx);
    })
        .then(function (auth) {
        if (auth !== true) {
            return ACCESS_DENIED;
        }
        return mutation.resolve(arg, mCtx.context, mCtx);
    })
        .then(function (value) {
        if (isAccessDenied(value)) {
            return {
                s: false,
                v: {
                    message: 'You do not have permission to call this mutation.',
                    code: 'AUTH_ERROR',
                },
            };
        }
        if (mutation.resultType.kind === "Void" /* Void */) {
            return { s: true, v: undefined };
        }
        validate_1.validateResult(mutation.resultType, value, mCtx.schema);
        return { s: true, v: value };
    })
        .catch(function (ex) {
        error_reporting_1.default(ex, mCtx.logging);
        if (process.env.NODE_ENV === 'production' && !ex.exposeProd) {
            return {
                s: false,
                v: {
                    message: 'Unexpected error while running ' +
                        mutation.name +
                        ' (If you are the software developer on this system, you can display ' +
                        'the actual error message by setting `NODE_ENV="development"` or setting ' +
                        'a property of `exposeProd = true` on the error object)',
                    code: 'PRODUCTION_ERROR',
                },
            };
        }
        return {
            s: false,
            v: {
                message: ex.message + ' while running ' + mutation.name,
                data: ex.data,
                code: ex.code || undefined,
            },
        };
    });
}
exports.default = runMutation;
//# sourceMappingURL=runMutation.js.map