"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var matchesType_1 = require("./matchesType");
var type_name_from_definition_1 = require("../utils/type-name-from-definition");
var type_name_from_value_1 = require("../utils/type-name-from-value");
var create_error_1 = require("../utils/create-error");
function validate(type, value, schema, options) {
    if (!matchesType_1.default(type, value, schema, false)) {
        var expected = type_name_from_definition_1.default(type);
        if (options.exposeValueInProduction ||
            process.env.NODE_ENV !== 'production') {
            var actual = type_name_from_value_1.default(value);
            throw create_error_1.default("Expected " + options.name + " to be of type \"" + expected + "\" but got \"" + actual + "\"", {
                exposeProd: options.exposeValueInProduction,
                code: options.code,
                data: { value: value, expected: expected },
            });
        }
        throw create_error_1.default("Expected " + options.name + " to be of type \"" + expected + "\"", {
            exposeProd: true,
            code: options.code,
            data: { expected: expected },
        });
    }
}
function createValidator(options) {
    return function (type, value, schema) { return validate(type, value, schema, options); };
}
exports.validateArg = createValidator({
    name: 'arg',
    exposeValueInProduction: true,
    code: 'INVALID_ARGUMENT_TYPE',
});
exports.validateResult = createValidator({
    name: 'result',
    exposeValueInProduction: false,
    code: 'INVALID_RESULT_TYPE',
});
//# sourceMappingURL=validate.js.map