/// <reference types="express" />
import { Request, Response, RequestHandler } from 'express';
import { FakeClient } from './server-rendering';
import SessionStore from './sessions/SessionStore';
import BicycleRequest from './types/Request';
import Logging from './types/Logging';
import MutationResult from './types/MutationResult';
import Query from './types/Query';
import Schema from './types/Schema';
import ServerPreparation from './types/ServerPreparation';
import ServerResponse from './types/ServerResponse';
import { BaseRootQuery, Mutation } from './typed-helpers/query';
import { Ctx } from './Ctx';
export interface Options {
    readonly disableDefaultLogging?: boolean;
    readonly sessionStore?: SessionStore;
    readonly onError?: (e: {
        error: Error;
    }) => any;
    readonly onMutationStart?: (e: {
        mutation: {
            readonly method: string;
            readonly args: Object;
        };
    }) => any;
    readonly onMutationEnd?: (e: {
        mutation: {
            readonly method: string;
            readonly args: Object;
        };
        result: MutationResult<any>;
    }) => any;
    readonly onQueryStart?: (e: {
        query: Object;
    }) => any;
    readonly onQueryEnd?: (e: {
        query: Object;
        cacheResult: Object;
    }) => any;
}
export default class BicycleServer<Context> {
    _logging: Logging;
    _schema: Schema<Context>;
    _sessionStore: SessionStore;
    constructor(schema: Schema<Context>, options?: Options);
    runQuery<TResult>(query: BaseRootQuery<TResult>, context: Ctx<Context>): Promise<TResult>;
    runQuery(query: Query, context: Ctx<Context>): Promise<any>;
    runMutation<TResult>(mutation: Mutation<TResult>, context: Ctx<Context>): Promise<TResult>;
    runMutation(method: string, args: any, context: Ctx<Context>): Promise<any>;
    handleMessage(message: BicycleRequest, getContext: (options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>): Promise<ServerResponse>;
    createMiddleware(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>): RequestHandler;
    createServerRenderer<TResult>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response) => TResult): (req: Request, res: Response) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, a1: TArg1) => TResult): (req: Request, a1: TArg1) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1, TArg2>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response, a1: TArg1, a2: TArg2) => TResult): (req: Request, res: Response, a1: TArg1, a2: TArg2) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1, TArg2, TArg3>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3) => TResult): (req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1, TArg2, TArg3, TArg4>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3, a4: TArg4) => TResult): (req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3, a4: TArg4) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
}
