"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var run_query_against_cache_1 = require("./utils/run-query-against-cache");
var runner_1 = require("./runner");
var handleMessage_1 = require("./handleMessage");
var middleware_1 = require("./middleware");
var server_rendering_1 = require("./server-rendering");
var MemorySessionStore_1 = require("./sessions/MemorySessionStore");
var query_1 = require("./typed-helpers/query");
var Ctx_1 = require("./Ctx");
function noop() {
    return null;
}
var BicycleServer = /** @class */ (function () {
    function BicycleServer(schema, options) {
        if (options === void 0) { options = {}; }
        this._schema = schema;
        this._sessionStore = options.sessionStore || new MemorySessionStore_1.default();
        this._logging = {
            disableDefaultLogging: options.disableDefaultLogging || false,
            onError: options.onError || noop,
            onMutationStart: options.onMutationStart || noop,
            onMutationEnd: options.onMutationEnd || noop,
            onQueryStart: options.onQueryStart || noop,
            onQueryEnd: options.onQueryEnd || noop,
        };
    }
    BicycleServer.prototype.runQuery = function (query, context) {
        var _this = this;
        var q = query instanceof query_1.BaseRootQuery ? query._query : query;
        return Ctx_1.default(context, function (context) {
            return runner_1.runQuery(q, {
                schema: _this._schema,
                logging: _this._logging,
                context: context,
            });
        }).then(function (cache) {
            var _a = run_query_against_cache_1.default(cache, q), loaded = _a.loaded, result = _a.result, errors = _a.errors;
            if (errors.length) {
                throw new Error(errors[0]);
            }
            if (!loaded) {
                throw new Error('Bicycle data failed to load, this should never happen.');
            }
            return result;
        });
    };
    BicycleServer.prototype.runMutation = function (method, args, context) {
        var _this = this;
        var m = method instanceof query_1.Mutation ? method._name : method;
        var a = method instanceof query_1.Mutation ? method._args : args;
        var c = method instanceof query_1.Mutation ? args : context;
        return Ctx_1.default(c, function (c) {
            return runner_1.runMutation({ method: m, args: a }, {
                schema: _this._schema,
                logging: _this._logging,
                context: c,
            });
        }).then(function (_a) {
            var s = _a.s, v = _a.v;
            if (s) {
                return v;
            }
            else {
                var err = new Error(v.message);
                err.data = v.data;
                err.code = v.code;
                throw err;
            }
        });
    };
    BicycleServer.prototype.handleMessage = function (message, getContext) {
        return handleMessage_1.default(this._schema, this._logging, this._sessionStore, message, function () { return getContext({ stage: 'query' }); }, function () { return getContext({ stage: 'mutation' }); });
    };
    BicycleServer.prototype.createMiddleware = function (getContext) {
        return middleware_1.default(this._schema, this._logging, this._sessionStore, getContext);
    };
    BicycleServer.prototype.createServerRenderer = function (getContext, fn) {
        return server_rendering_1.default(this._schema, this._logging, this._sessionStore, getContext, fn);
    };
    return BicycleServer;
}());
exports.default = BicycleServer;
//# sourceMappingURL=server-core.js.map