"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var cuid = require("cuid");
var not_equal_1 = require("./utils/not-equal");
var merge_queries_1 = require("./utils/merge-queries");
var run_query_against_cache_1 = require("./utils/run-query-against-cache");
var get_session_id_1 = require("./utils/get-session-id");
var runner_1 = require("./runner");
var ServerPreparation_1 = require("./types/ServerPreparation");
var query_1 = require("./typed-helpers/query");
var Ctx_1 = require("./Ctx");
var FakeClient = /** @class */ (function () {
    function FakeClient(sessionID) {
        this._sessionID = sessionID;
        this._query = {};
        this._cache = { Root: { root: {} } };
    }
    FakeClient.prototype.queryCache = function (query) {
        var q = query instanceof query_1.BaseRootQuery ? query._query : query;
        this._query = merge_queries_1.default(this._query, q);
        return run_query_against_cache_1.default(this._cache, q);
    };
    FakeClient.prototype.update = function () {
        throw new Error('Bicycle server renderer does not implement update');
    };
    FakeClient.prototype.subscribe = function () {
        throw new Error('Bicycle server renderer does not implement subscribe');
    };
    return FakeClient;
}());
exports.FakeClient = FakeClient;
function prepare(schema, logging, sessionStore, getContext, fn) {
    return function (req, res) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        return Promise.all([
            get_session_id_1.default(sessionStore),
            getContext(req, res, { stage: 'query' }),
        ]).then(function (_a) {
            var sessionID = _a[0], context = _a[1];
            var client = new FakeClient(sessionID);
            return Ctx_1.default(context, function (context) {
                var queryContext = { schema: schema, logging: logging, context: context };
                return new Promise(function (resolve, reject) {
                    function next() {
                        try {
                            var oldQuery = client._query;
                            var oldCache_1 = client._cache;
                            var result_1 = fn.apply(void 0, [client, req, res].concat(args));
                            var newQuery = client._query;
                            if (!not_equal_1.default(oldQuery, newQuery)) {
                                return resolve(result_1);
                            }
                            runner_1.runQuery(newQuery, queryContext).then(function (data) {
                                if (not_equal_1.default(oldCache_1, data)) {
                                    client._cache = data;
                                    return next();
                                }
                                else {
                                    return resolve(result_1);
                                }
                            }, function (err) {
                                reject(err);
                            });
                        }
                        catch (ex) {
                            reject(ex);
                        }
                    }
                    next();
                }).then(function (result) {
                    return sessionStore
                        .tx(sessionID, function () {
                        var version = cuid();
                        return Promise.resolve({
                            session: {
                                versions: [
                                    { version: version, query: client._query, cache: client._cache },
                                ],
                                mutations: {},
                            },
                            result: version,
                        });
                    })
                        .then(function (version) { return ({
                        result: result,
                        serverPreparation: ServerPreparation_1.createServerPreparation(sessionID, version, client._query, client._cache),
                    }); });
                });
            });
        });
    };
}
exports.default = prepare;
//# sourceMappingURL=server-rendering.js.map