"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var HashLRU = /** @class */ (function () {
    function HashLRU(max) {
        this._size = 0;
        this._previous = Object.create(null);
        this._current = Object.create(null);
        this._max = max;
        if (typeof max !== 'number' || max < 1 || (max | 0) !== max) {
            throw Error('hashlru must have a max value, of type number, greater than 0');
        }
    }
    HashLRU.prototype._update = function (key, value) {
        this._current[key] = value;
        this._size++;
        if (this._size >= this._max) {
            this._size = 0;
            this._previous = this._current;
            this._current = Object.create(null);
        }
        else if (this._previous[key] !== undefined) {
            this._previous[key] = undefined;
        }
    };
    HashLRU.prototype.has = function (key) {
        return (this._previous[key] !== undefined || this._current[key] !== undefined);
    };
    HashLRU.prototype.remove = function (key) {
        if (this._current[key] !== undefined)
            this._current[key] = undefined;
        if (this._previous[key] !== undefined)
            this._previous[key] = undefined;
    };
    HashLRU.prototype.get = function (key) {
        var v = this._current[key];
        if (v !== undefined)
            return v;
        if ((v = this._previous[key]) !== undefined) {
            this._update(key, v);
            return v;
        }
    };
    HashLRU.prototype.set = function (key, value) {
        if (this._current[key] !== undefined)
            this._current[key] = value;
        else
            this._update(key, value);
    };
    HashLRU.prototype.clear = function () {
        this._current = Object.create(null);
        this._previous = Object.create(null);
    };
    return HashLRU;
}());
exports.default = HashLRU;
//# sourceMappingURL=HashLRU.js.map