import Q from '../types/Query';
import { OptimisticUpdateHandler } from '../client/optimistic';
declare const stringify: (value: any) => string;
export { stringify };
export { Q };
export declare function merge(a: Q, b: Q): Q;
export declare function addField(query: Q, field: string, subQuery: true | Q): Q;
export declare abstract class BaseQuery<TResult> {
    readonly _query: Q;
    /**
     * Usage:
     *   type Result = typeof Query.$type;
     */
    readonly $type: TResult;
    constructor(query: Q);
}
export declare abstract class BaseRootQuery<TResult> extends BaseQuery<TResult> {
    protected _root: true;
}
export declare class Mutation<TResult> {
    readonly _name: string;
    readonly _args: any;
    readonly _optimisticUpdate: OptimisticUpdateHandler | void;
    /**
     * Usage:
     *   type Result = typeof Mutation.$type;
     */
    readonly $type: TResult;
    constructor(name: string, args: any, optimisticUpdate: OptimisticUpdateHandler | void);
}
/**
 * Usage:
 *
 *     type TResult = typeof getType(query);
 *
 * @param query A bicycle query
 */
declare function getType<TResult>(query: BaseQuery<TResult>): TResult;
declare function getType<TResult>(query: Mutation<TResult>): TResult;
declare function getType<TResult>(query: (...args: any[]) => Mutation<TResult>): TResult;
export { getType };
