import MutationResult from './MutationResult';
import SessionID from './SessionID';
import SessionVersion from './SessionVersion';
import Cache, { CacheUpdate } from './Cache';
export declare const enum ServerResponseKind {
    EXPIRED = 0,
    NEW_SESSION = 1,
    UPDATE = 2,
    RESTORE = 3,
}
export interface ServerResponseExpired {
    readonly k: ServerResponseKind.EXPIRED;
    readonly m: void | MutationResult<any>[];
}
export interface ServerResponseNewSession {
    readonly k: ServerResponseKind.NEW_SESSION;
    readonly s: SessionID;
    readonly v: SessionVersion;
    readonly d: Cache;
}
export interface ServerResponseUpdate {
    readonly k: ServerResponseKind.UPDATE;
    readonly v: SessionVersion;
    readonly d: void | CacheUpdate;
    readonly m: void | MutationResult<any>[];
}
export interface ServerResponseRestore {
    readonly k: ServerResponseKind.RESTORE;
    readonly v: SessionVersion;
    readonly d: Cache;
    readonly m: void | MutationResult<any>[];
}
declare type ServerResponse = ServerResponseExpired | ServerResponseNewSession | ServerResponseUpdate | ServerResponseRestore;
export default ServerResponse;
export declare function createExpiredSessionResponse(mutationResults: void | MutationResult<any>[]): ServerResponseExpired;
export declare function createNewSessionResponse(sessionID: SessionID, version: SessionVersion, data: Cache): ServerResponseNewSession;
export declare function createUpdateResponse(version: SessionVersion, data: void | CacheUpdate, mutationResults: void | MutationResult<any>[]): ServerResponseUpdate;
export declare function createRestoreResponse(version: SessionVersion, data: Cache, mutationResults: void | MutationResult<any>[]): ServerResponseRestore;
