"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Cache_1 = require("../types/Cache");
function deepEqual(a, b) {
    if (a === b) {
        return true;
    }
    if (Array.isArray(a) && Array.isArray(b)) {
        return a.length === b.length && a.every(function (val, i) { return deepEqual(val, b[i]); });
    }
    if (a && b && typeof a === 'object' && typeof b === 'object') {
        var aKeys = Object.keys(a);
        var bKeys = Object.keys(b);
        return (aKeys.length === bKeys.length &&
            aKeys.every(function (key) { return key in b && deepEqual(a[key], b[key]); }));
    }
    return false;
}
function diffCacheObject(before, after) {
    var result = undefined;
    Object.keys(after).forEach(function (key) {
        var beforeProp = before[key];
        var afterProp = after[key];
        if (beforeProp === afterProp)
            return;
        if (Array.isArray(beforeProp) &&
            Array.isArray(afterProp) &&
            beforeProp.length === afterProp.length &&
            beforeProp.every(function (val, i) { return deepEqual(val, afterProp[i]); }))
            return;
        if (Cache_1.isCacheObject(beforeProp) && Cache_1.isCacheObject(afterProp)) {
            var d = diffCacheObject(beforeProp, afterProp);
            if (d) {
                if (!result)
                    result = {};
                result[key] = d;
            }
            return;
        }
        if (!result)
            result = {};
        result[key] = afterProp;
    });
    return result;
}
function diffNodeCache(before, after) {
    var result = undefined;
    Object.keys(after).forEach(function (key) {
        var beforeValue = before[key];
        var afterValue = after[key];
        if (afterValue) {
            if (beforeValue) {
                var d = diffCacheObject(beforeValue, afterValue);
                if (d) {
                    if (!result)
                        result = {};
                    result[key] = d;
                }
            }
            else {
                if (!result)
                    result = {};
                result[key] = afterValue;
            }
        }
    });
    return result;
}
function diffCache(before, after) {
    var result = undefined;
    Object.keys(after).forEach(function (key) {
        var beforeValue = before[key];
        var afterValue = after[key];
        if (afterValue) {
            if (beforeValue) {
                var d = diffNodeCache(beforeValue, afterValue);
                if (d) {
                    if (!result)
                        result = {};
                    result[key] = d;
                }
            }
            else {
                if (!result)
                    result = {};
                result[key] = afterValue;
            }
        }
    });
    return result;
}
exports.default = diffCache;
//# sourceMappingURL=diff-cache.js.map