"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ErrorResult_1 = require("../types/ErrorResult");
var NodeID_1 = require("../types/NodeID");
function isCached(cache, nodeID, key, query) {
    function resolveValue(value, subQuery) {
        if (value === undefined) {
            return false;
        }
        else if (ErrorResult_1.isErrorResult(value)) {
            return true;
        }
        else if (NodeID_1.isID(value)) {
            if (subQuery === true) {
                return true;
            }
            else {
                return recurse(NodeID_1.getNodeIfExists(cache, value), subQuery);
            }
        }
        else if (Array.isArray(value)) {
            return value.every(function (v) { return resolveValue(v, subQuery); });
        }
        else if (value && typeof value === 'object') {
            return Object.keys(value).every(function (key) {
                return resolveValue(value[key], subQuery);
            });
        }
        return true;
    }
    function recurse(node, query) {
        if (!node) {
            return false;
        }
        return Object.keys(query).every(function (key) {
            if (key[0] === '_')
                return true;
            var cacheKey = key.replace(/ as [a-zA-Z0-9]+$/, '');
            var value = node[cacheKey];
            var subQuery = query[key];
            return resolveValue(value, subQuery);
        });
    }
    var entryNode = NodeID_1.getNodeIfExists(cache, nodeID);
    if (!entryNode) {
        return false;
    }
    return resolveValue(entryNode[key], query);
}
exports.default = isCached;
//# sourceMappingURL=is-cached.js.map