"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ErrorResult_1 = require("../types/ErrorResult");
var NodeID_1 = require("../types/NodeID");
var freeze_1 = require("./freeze");
var EMPTY_ARRAY = freeze_1.default([]);
var spareArray = [];
var spareDetailsArray = [];
var EMPTY_OBJECT = {};
function getRoot(cache) {
    var c = cache.Root;
    if (!c)
        return EMPTY_OBJECT;
    return c.root || EMPTY_OBJECT;
}
function runQueryAgainstCache(cache, query) {
    var loaded = true;
    var errors = spareArray;
    var errorDetails = spareDetailsArray;
    function resolveValue(value, subQuery) {
        if (value === undefined) {
            loaded = false;
            return undefined;
        }
        else if (ErrorResult_1.isErrorResult(value)) {
            if (errors.indexOf(value.message) === -1) {
                errors.push(value.message);
            }
            errorDetails.push(value);
            return value;
        }
        else if (NodeID_1.isID(value)) {
            if (subQuery === true) {
                return {};
            }
            else {
                return recurse(NodeID_1.getNode(cache, value), subQuery);
            }
        }
        else if (Array.isArray(value)) {
            return value.map(function (v) { return resolveValue(v, subQuery); });
        }
        else if (value && typeof value === 'object') {
            var result_1 = {};
            Object.keys(value).forEach(function (key) {
                result_1[key] = resolveValue(value[key], subQuery);
            });
            return result_1;
        }
        else {
            return value;
        }
    }
    function recurse(node, query) {
        var result = {};
        Object.keys(query).forEach(function (key) {
            if (key[0] === '_')
                return;
            var cacheKey = key.replace(/ as [a-zA-Z0-9]+$/, '');
            var resultKey = / as [a-zA-Z0-9]+$/.test(key)
                ? key.split(' as ').pop()
                : cacheKey.split('(')[0];
            var value = node[cacheKey];
            var subQuery = query[key];
            result[resultKey] = resolveValue(value, subQuery);
        });
        return result;
    }
    var result = recurse(getRoot(cache), query);
    if (errors.length === 0) {
        errors = EMPTY_ARRAY;
        errorDetails = EMPTY_ARRAY;
    }
    else {
        spareArray = [];
        spareDetailsArray = [];
        errors = freeze_1.default(errors);
        errorDetails = freeze_1.default(errorDetails);
    }
    return { result: result, loaded: loaded, errors: errors, errorDetails: errorDetails };
}
exports.default = runQueryAgainstCache;
//# sourceMappingURL=run-query-against-cache.js.map