"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("util");
function typeNameFromDefinition(type) {
    switch (type.kind) {
        case "Boolean" /* Boolean */:
            return 'boolean';
        case "List" /* List */:
            var needsBrackets = type.element.kind === "Union" /* Union */;
            return ((needsBrackets ? '(' : '') +
                typeNameFromDefinition(type.element) +
                (needsBrackets ? ')' : '') +
                '[]');
        case "Literal" /* Literal */:
            return util_1.inspect(type.value);
        case "Named" /* Named */:
            return type.name;
        case "Null" /* Null */:
            return 'null';
        case "Any" /* Any */:
            return 'any';
        case "Number" /* Number */:
            return 'number';
        case "Object" /* Object */:
            return ('{' +
                Object.keys(type.properties)
                    .sort()
                    .map(function (p) { return p + ': ' + typeNameFromDefinition(type.properties[p]); })
                    .join(', ') +
                '}');
        case "Promise" /* Promise */:
            return 'Promise<' + typeNameFromDefinition(type.result) + '>';
        case "String" /* String */:
            return 'string';
        case "Union" /* Union */:
            return type.elements
                .map(function (e) { return typeNameFromDefinition(e); })
                .sort()
                .join(' | ');
        case "Void" /* Void */:
            return 'void';
    }
}
exports.default = typeNameFromDefinition;
//# sourceMappingURL=type-name-from-definition.js.map